package com.htfp.weather.download.gfs;

import com.fasterxml.jackson.annotation.JsonIgnore;
import com.htfp.weather.utils.JSONUtils;
import com.htfp.weather.web.exception.AppException;
import com.htfp.weather.web.exception.ErrorCode;
import lombok.Data;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.io.IOUtils;
import org.springframework.stereotype.Component;

import javax.annotation.PostConstruct;
import javax.validation.Valid;
import javax.validation.constraints.Max;
import javax.validation.constraints.Min;
import javax.validation.constraints.NotEmpty;
import javax.validation.constraints.NotNull;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.nio.file.NoSuchFileException;
import java.util.ArrayList;

/**
 * @Author : shiyi
 * @Date : 2024/2/28 13:53
 * @Description : GFS数据下载配置
 */
@Slf4j @Data
@Component
public class GfsDataConfig {
    @NotNull(message = "下载范围 (duration) 不能为空")
    @Min(value = 1, message = "下载范围 (duration) 需大于等于1小时")
    @Max(value = 36, message = "下载范围 (duration) 不能超过36小时")
    private int duration = 5;

    @NotNull(message = "最小经度不能为空")
    @Min(value = -180, message = "经度最小值为-180（西经180°）")
    @Max(value = 180, message = "经度最大值为180（东经180°）")
    private Double minLon;

    @NotNull(message = "最大经度不能为空")
    @Min(value = -180, message = "经度最小值为-180（西经180°）")
    @Max(value = 180, message = "经度最大值为180（东经180°）")
    private Double maxLon;

    @NotNull(message = "最小纬度不能为空")
    @Min(value = -90, message = "纬度最小值为-90（南纬90°）")
    @Max(value = 90, message = "纬度最大值为90（北纬90°）")
    private Double minLat;

    @NotNull(message = "最大纬度不能为空")
    @Min(value = -90, message = "纬度最小值为-90（南纬90°）")
    @Max(value = 90, message = "纬度最大值为90（北纬90°）")
    private Double maxLat;

    @NotNull(message = "分辨率 (resolution) 不能为空")
    private Double resolution = 0.25;

    @Valid
    @NotEmpty(message = "变量列表 (variables) 不能为空")
    // @EnumValid(enumClass = GfsDataVariableEnum.class, message = "变量列表 (variables) 必须为 GfsDataVariableEnum 枚举值")
    private ArrayList<String> variables;

    @Valid
    @NotEmpty(message = "气压高度列表 (pressureLevels) 不能为空")
    // @EnumValid(enumClass = GfsLevelsEnum.class, message = "变量列表 (pressureLevels) 必须为 GfsLevelsEnum 枚举值")
    private int[] pressureLevels;
    private int[] heightLevels = new int[]{2, 10, 20, 30, 40, 50, 80, 100};
    @NotEmpty(message = "数据存储路径 (saveRoot) 不能为空")
    private String saveRoot;

    @JsonIgnore
    // private final String configPath = Objects.requireNonNull(this.getClass().getClassLoader().getResource("config")).getPath() + "/gfsDataConfig.json";
    private final String configPath = System.getProperty("user.dir") + "/gfsDataConfig.json";
    public void valid() {
        // TODO 2024/4/21: 校验
        if (minLon >= maxLon) {
            throw new AppException(ErrorCode.VALIDATE_ERROR, "最小经度必须小于最大经度");
        }
        if (minLat >= maxLat) {
            throw new AppException(ErrorCode.VALIDATE_ERROR, "最小纬度必须小于最大纬度度");
        }
        ArrayList<String> wrongVariables = new ArrayList<>();
        ArrayList<Integer> wrongLevels = new ArrayList<>();
        for (String variable : variables) {
            if (!GfsDownloadVariableEnum.contains(variable)) {
                wrongVariables.add(variable);
            }
        }
        if (!wrongVariables.isEmpty()) {
            throw new AppException(ErrorCode.VALIDATE_ERROR, " variables = " +wrongVariables + "非 GfsLevelsEnum 枚举值");
        }

        for (Integer level : pressureLevels) {
            if (!GfsLevelsEnum.contains(level)) {
                wrongLevels.add(level);
            }
        }
        if (!wrongLevels.isEmpty()) {
            throw new AppException(ErrorCode.VALIDATE_ERROR, " pressureLevels = " +wrongLevels + "非 GfsLevelsEnum 枚举值");
        }
    }
    @PostConstruct
    public void init() {
        readConfig();
    }
    public void readConfig() {
        try (InputStream f = new FileInputStream(configPath)){
            GfsDataConfig gfsDataConfig = JSONUtils.json2pojo(IOUtils.toString(f), GfsDataConfig.class);
            this.duration = gfsDataConfig.getDuration();
            this.minLon = gfsDataConfig.getMinLon();
            this.maxLon = gfsDataConfig.getMaxLon();
            this.minLat = gfsDataConfig.getMinLat();
            this.maxLat = gfsDataConfig.getMaxLat();
            this.resolution = gfsDataConfig.getResolution();
            this.variables = gfsDataConfig.getVariables();
            this.pressureLevels = gfsDataConfig.getPressureLevels();
            this.saveRoot = gfsDataConfig.getSaveRoot();
            log.info("[config] 下载配置初始化: {}", configPath);
        } catch (NoSuchFileException e) {
            log.error("[config] 配置文件{}不存在", configPath, e);
        } catch (Exception ex) {
            throw new RuntimeException(ex);
        }
    }


    public void writeConfig(GfsDataConfig updateConfig) throws IOException {
        writeConfig(updateConfig, configPath);
    }
    public void writeConfig(GfsDataConfig updateConfig, String filePath) throws IOException {
        this.duration = updateConfig.getDuration();
        this.minLon = updateConfig.getMinLon();
        this.maxLon = updateConfig.getMaxLon();
        this.minLat = updateConfig.getMinLat();
        this.maxLat = updateConfig.getMaxLat();
        this.resolution = updateConfig.getResolution();
        this.variables = updateConfig.getVariables();
        this.pressureLevels = updateConfig.getPressureLevels();
        this.saveRoot = updateConfig.getSaveRoot();
        // 如果目录不存在则创建
        File destDir = new File(saveRoot);
        if (!destDir.exists()) {
            boolean mkdirs = destDir.mkdirs();
            if (!mkdirs) {
                throw new AppException(ErrorCode.CONFIG_ERROR, "目录创建失败: " + saveRoot);
            }
        }
        JSONUtils.pojo2jsonFile(this, filePath);
        log.info("配置文件 {} 更新为: {}", filePath, updateConfig);
    }
}
