package com.htfp.weather.griddata.operation;

import com.aliyun.tablestore.grid.GridDataWriter;
import com.aliyun.tablestore.grid.model.GridDataSetMeta;
import com.aliyun.tablestore.grid.model.StoreOptions;
import com.aliyun.tablestore.grid.model.grid.Grid2D;

import com.htfp.weather.download.gfs.GfsVariableHeightEnum;
import com.htfp.weather.download.gfs.GfsVariableIsobaricEnum;
import com.htfp.weather.griddata.common.TableConfig;
import com.htfp.weather.info.Constant;
import com.htfp.weather.utils.MeteoUtils;
import com.htfp.weather.web.exception.AppException;
import com.htfp.weather.web.exception.ErrorCode;
import lombok.Data;
import lombok.Getter;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.context.annotation.DependsOn;
import org.springframework.stereotype.Component;
import org.springframework.util.CollectionUtils;
import ucar.ma2.*;
import ucar.nc2.NetcdfFile;
import ucar.nc2.NetcdfFiles;
import ucar.nc2.Variable;

import javax.annotation.PreDestroy;
import javax.annotation.Resource;
import java.io.File;
import java.time.OffsetDateTime;
import java.time.format.DateTimeFormatter;
import java.util.*;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.stream.Collectors;

/**
 * @author shi_y
 */
@Slf4j
@Component
@DependsOn("tableConfig")
public class GfsDataImport extends BaseTableOperation {
    @Resource
    TableConfig tableConfig;
    @Resource
    GfsDataFetcher gfsDataFetcher;
    @Getter
    boolean importing = false;
    private final ExecutorService executorService = Executors.newFixedThreadPool(5);

    /**
     * 表格元数据初始化
     *
     * @param dataSetID
     * @param dataType
     * @param variables
     * @param shape     // * @param attribute meta表的属性列
     * @return
     * @throws Exception
     */
    private GridDataSetMeta initMeta(String dataSetID, DataType dataType, List<String> variables, int[] shape) throws Exception {
        GridDataSetMeta meta = new GridDataSetMeta(
                dataSetID,
                dataType,
                variables,
                shape[0],
                shape[1],
                shape[2],
                shape[3],
                new StoreOptions(StoreOptions.StoreType.SLICE));
        // for (Map.Entry<String, Object> entry : attributes.entrySet()) {
        //     meta.addAttribute(entry.getKey(), entry.getValue());
        // }
        meta.addAttribute("status", "INIT");
        meta.addAttribute("create_time", System.currentTimeMillis());
        tableStoreGrid.putDataSetMeta(meta);
        return meta;
    }

    /**
     * 导入完成后，更新meta状态
     *
     * @param meta
     * @return
     * @throws Exception
     */
    public GridDataSetMeta updateMeta(GridDataSetMeta meta) throws Exception {
        tableStoreGrid.updateDataSetMeta(meta);
        return meta;
    }

    /**
     * 导入完成后，设置meta状态
     */
    public GridDataSetMeta putMeta(GridDataSetMeta meta) throws Exception {
        tableStoreGrid.putDataSetMeta(meta);
        return meta;
    }

    /** 导入指定起报时间的所有文件*/
    public synchronized List<ImportFileInfo> importData(OffsetDateTime refTime) throws Exception {
        importing = true;
        try {
            long start = System.currentTimeMillis();
            List<String> fileList = getFiles(refTime);
            if (CollectionUtils.isEmpty(fileList)) {
                throw new AppException(ErrorCode.NO_NC_OR_GRIB_FILES);
            }
            log.info("[tablestore] 数据导入开始, refTime = {}...", refTime);
            // datasetId和起报时间绑定
            String dataSetId = refTime.format(DateTimeFormatter.ofPattern(Constant.DATA_SET_ID_FORMAT_STRING));
            List<String> fileVariables = getFileVariables(tableConfig.variableList);
            int[] shape = new int[]{tableConfig.timeSizeMax, tableConfig.levSize, tableConfig.latSize, tableConfig.lonSize};
            GridDataSetMeta meta = initMeta(dataSetId, tableConfig.dataType, fileVariables, shape);
            List<String> forecastHours = new ArrayList<>(); // 所有有效预报时效坐标记录到数据库属性中

            // todo 2024/5/13: 待优化，用于数据库的索引，必须连续，因此必须保证前一个时刻成功导入后才能导入下一个时刻，数据量大的时候导入时间较长
            List<ImportFileInfo> finishedList;
            // List<Future<ImportResult>> futures = new ArrayList<>();
            finishedList = new ArrayList<>();
            for (int i = 0; i < fileList.size(); i++) {
                String file = fileList.get(i);
                int iTime = i;
                int forecastHour = getForecastHourFromFilename(file);
                // 2024/5/13:使用多线程，并使用Future来获取结果，但是实际瓶颈可能是网络带宽而非计算
                // futures.add(executorService.submit(() -> importFromNcFile(meta, file, iTime,forecastHour)));
                ImportFileInfo importFileInfo = importFromNcFile(meta, file, iTime, forecastHour);
                finishedList.add(importFileInfo);
                if (importFileInfo.isSuccess()) {
                    forecastHours.add(String.valueOf(importFileInfo.getForecastHour()));
                }
            }
            // for (Future<ImportResult> future : futures) {
            //     ImportResult importResult = future.get();
            //     finishedList.add(importResult);
            //     forecastHours.add(String.valueOf(importResult.getForecastHour()));
            //     if (importResult.isSuccess()) {
            //         forecastHours.add(String.valueOf(importResult.getForecastHour()));
            //     }
            // }

            List<ImportFileInfo> failedList = finishedList.stream().filter(result -> !result.isSuccess()).collect(Collectors.toList());
            if (!failedList.isEmpty()) {
                log.warn("存在气象数据导入数据库失败，失败文件列表： {} ", failedList);
                meta.addAttribute("status", "WRONG");
            } else {
                meta.addAttribute("status", "DONE");
            }
            long end = System.currentTimeMillis();
            log.info("[tablestore] 数据导入完成, 耗时: {} s, forecastHours: {}", (end - start)/1000., forecastHours);
            meta.setForecastHours(forecastHours);
            meta.addAttribute("reference_time", refTime.toInstant().toEpochMilli());
            putMeta(meta);
            gfsDataFetcher.lastGridDataSetMeta = meta;
            log.info("[tablestore]: 更新最新的数据元信息: {}", meta);
            return finishedList;
        } finally {
            importing = false;
        }
    }

    /**
     * 读取grib/nc文件，并进行相关计算，最后导入tablestore数据库
     * @param meta  数据元信息
     * @param file  netcdf文件路径
     * @param iTime 文件序号
     * @param forecastHour 预报时效（相对于起报时间的小时数）
     */
    public ImportFileInfo importFromNcFile(GridDataSetMeta meta, String file, int iTime, int forecastHour) {
        try (NetcdfFile ncFile = NetcdfFiles.open(file)) {
            GridDataWriter writer = tableStoreGrid.getDataWriter(meta);

            // 相对reftime的小时数，文件可能缺失，因此time可能是不连续的，但是iTime是连续的
            if (forecastHour == -1) {
                forecastHour = (int) ncFile.findVariable("time").read().getDouble(0);
            }
            int xsize = meta.getxSize();
            int ysize = meta.getySize();

            for (String variableName : meta.getVariables()) {
                // 风速在最后单独处理
                if ("Wind_speed_gust_surface".equals(variableName) || "Wind_speed_isobaric".equals(variableName) || "Wind_direction_isobaric".equals(variableName)
                    || "Wind_speed_height_above_ground".equals(variableName) || "Wind_direction_height_above_ground".equals(variableName)) {
                    continue;
                }
                Variable variable = ncFile.findVariable(variableName);
                if (variable != null) {
                    int shapeLength = variable.getShape().length;
                    if (shapeLength== 4) { // 存在垂直坐标，即变量随高度变化
                        // 气压坐标和高度坐标保持各自的索引
                        int zSize = variable.getShape(1);
                        for (int z = 0; z < zSize; z++) {
                            // 注意高度索引是递增排序
                            Array array = variable.read(new int[]{0, z, 0, 0}, new int[]{1, 1, xsize, ysize});
                            transferUnit(variableName, array);
                            Grid2D grid2D = new Grid2D(array.getDataAsByteBuffer(), variable.getDataType(),
                                    new int[]{0, 0}, new int[]{xsize, ysize});
                            writer.writeGrid2D(variableName, iTime, z, grid2D);
                        }
                    } else if (shapeLength == 3) {
                        // DONE 2024/5/24: 导入不含高度坐标的数据，如降水量，阵风风速
                        Array array = variable.read(new int[]{0, 0, 0}, new int[]{1,xsize, ysize});
                        transferUnit(variableName, array);
                        Grid2D grid2D = new Grid2D(array.getDataAsByteBuffer(), variable.getDataType(),
                                new int[]{0, 0}, new int[]{xsize, ysize});
                        writer.writeGrid2D(variableName, iTime, 0, grid2D);
                    }
                } else {
                    log.warn("[tablestore] 数据文件 {} 中没有变量 {}", ncFile.getLocation(), variableName);
                }
            }
            // 导入风速风向
            importWind(writer, meta, ncFile, iTime);
        } catch (Exception e) {
            log.error("[tablestore] 导入文件数据失败,_t={}: {}", forecastHour, file, e);
            return new ImportFileInfo(false, file, forecastHour, iTime);
        }
        log.info("[tablestore] 导入文件数据成功,_t={}: {}", forecastHour, file);
        return new ImportFileInfo(true, file, forecastHour, iTime);
    }

    /**单位转换*/
    private void transferUnit(String variableName, Array array) {
        if (GfsVariableIsobaricEnum.TEMP.getNameInFile().equals(variableName)) {
            MeteoUtils.kelvin2Celsius(array);
        }
        if (GfsVariableHeightEnum.TEMP.getNameInFile().equals(variableName)) {
            MeteoUtils.kelvin2Celsius(array);
        }
        if (GfsVariableHeightEnum.PRECIP.getNameInFile().equals(variableName)) {
            MeteoUtils.precipRate2mmPerHour(array);
        }
    }

    /** 计算风速风向，包括地面坐标和气压坐标，并导入数据库*/
    private void importWind(GridDataWriter writer, GridDataSetMeta meta, NetcdfFile ncFile, int iTime) throws Exception {
        // TODO 2024/5/8: 风速风向最好单独保存为一个文件，如果后续需要二维可视化，可以直接本地读取出图，减少网络请求
        for (String suffix : new String[]{"_isobaric", "_height_above_ground"}) {
            Variable uwnd = ncFile.findVariable("u-component_of_wind" + suffix);
            Variable vwnd = ncFile.findVariable("v-component_of_wind" + suffix);
            if (uwnd == null || vwnd == null) {
                return;
            }
            int xsize = meta.getxSize();
            int ysize = meta.getySize();
            int zsize = uwnd.getShape(1);
            for (int z = 0; z < zsize; z++) {
                Array uwndArray = uwnd.read(new int[]{0, z, 0, 0}, new int[]{1, 1, xsize, ysize});
                Array vwndArray = vwnd.read(new int[]{0, z, 0, 0}, new int[]{1, 1, xsize, ysize});

                Array speedArray = MeteoUtils.calculateWindSpeed(uwndArray, vwndArray);
                Array dirArray = MeteoUtils.calculateWindDirection(uwndArray, vwndArray);
                Grid2D speedGrid2D = new Grid2D(speedArray.getDataAsByteBuffer(), speedArray.getDataType(),
                        new int[]{0, 0}, new int[]{xsize, ysize});
                writer.writeGrid2D("Wind_speed" + suffix, iTime, z, speedGrid2D);

                Grid2D dirGrid2D = new Grid2D(dirArray.getDataAsByteBuffer(), dirArray.getDataType(),
                        new int[]{0, 0}, new int[]{xsize, ysize});
                writer.writeGrid2D("Wind_direction" + suffix, iTime, z, dirGrid2D);
            }
        }
    }
    /** 从文件名中获取预报时效 */
    private int getForecastHourFromFilename(String filename) {
        try {
            String s = filename.split("\\+")[1].split(".grib2")[0];
            if (StringUtils.isEmpty(s)) {
                return -1;
            } else {
                return Integer.parseInt(s);
            }
        } catch (Exception e) {
            return -1;
        }
    }
    private List<String> getFileVariables(List<String> variables) {
        List<String> fileVariables = new ArrayList<>();
        for (String variable : variables) {
            String variableIsobaric = GfsVariableIsobaricEnum.getGfsVariableName(variable);
            if (variableIsobaric != null) {
                fileVariables.add(variableIsobaric);
            } else {
                log.warn("GFS数据文件中（气压坐标）没有 {} 对应的变量", variable);
            }

            String variableAboveGround = GfsVariableHeightEnum.getGfsVariableName(variable);
            if (variableAboveGround != null) {
                fileVariables.add(variableAboveGround);
            } else {
                log.warn("GFS数据文件中（高度坐标）没有 {} 对应的变量", variable);
            }
        }
        return fileVariables;
    }

    /**
     * 获取文件目录中的所有文件绝对路径
     *
     * @param refTime，UTC起报时间
     * @return
     */
    private List<String> getFiles(OffsetDateTime refTime) {
        String dataFolder = refTime.format(DateTimeFormatter.ofPattern(Constant.DATA_FOLDER_STRING));
        File fileDir = new File(tableConfig.dataDir, dataFolder);
        if (!fileDir.exists()) {
            log.warn("文件夹 {} 不存在", fileDir);
            return null;
        }
        List<String> fileList = new ArrayList<>();
        String[] files = fileDir.list((dir, name) -> {
            return name.startsWith("BJT") && name.endsWith("grib2"); // 判断文件名是否以“BJT”开头
        });
        if (files == null || files.length == 0) {
            log.warn("文件夹 {} 中没有符合条件的文件", fileDir);
            return fileList;
        }
        for (String filename : files) {
            fileList.add(fileDir.getAbsolutePath() + File.separator + filename);
        }
        return fileList;
    }

    @PreDestroy
    public void end() {
        close();
    }

    @Data
    public static class ImportFileInfo {
        private boolean success;
        private String file;
        private int forecastHour;
        private int iTime;

        public ImportFileInfo(boolean success, String file, int forecastHour, int iTime) {
            this.success = success;
            this.file = file;
            this.forecastHour = forecastHour;
            this.iTime = iTime;
        }
    }

}
