package com.htfp.weather.schedule;

import com.htfp.weather.download.DownLoadFileInfo;
import com.htfp.weather.download.gfs.GfsDataConfig;
import com.htfp.weather.download.gfs.GfsDownloader;
import com.htfp.weather.griddata.operation.GfsDataImport;
import com.htfp.weather.griddata.operation.GfsDataImport.ImportFileInfo;
import com.htfp.weather.info.Constant;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.io.FileUtils;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.DependsOn;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.io.File;
import java.io.IOException;
import java.time.LocalDateTime;
import java.time.OffsetDateTime;
import java.time.ZoneOffset;
import java.time.format.DateTimeFormatter;
import java.util.List;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.Future;
import java.util.stream.Collectors;

/**
 * @Author : shiyi
 * @Date : 2024/5/8 11:36
 * @Description : 更新数据库数据，包括下载、导入、清楚本地过期数据
 */
@Component
@Slf4j
@DependsOn({"gfsDataImport", "gfsDownloader"})
public class GridDataProcessor {
    @Resource
    GfsDataImport gfsDataImport;
    @Resource
    GfsDownloader gfsDownloader;
    @Resource
    GfsDataConfig gfsDataConfig;
    @Resource
    ErrorReportService errorReportService;

    @Value("${mail.send.to}")
    private String[] mailReceiver;

    private final ExecutorService executorService = Executors.newFixedThreadPool(3);
    boolean test = false;

    /**每日北京时0,8,12,18点执行*/
    @Scheduled(cron = "0 0 0,8,12,18 * * ?")
    public void dailyDataProcess() {
        if (download()) {
            importToTableStore();
        }
        clearExpiredData();
    }

    private boolean download() {
        gfsDownloader.iniTimeSetting();
        try {
            log.info("[schedule-start] 开始下载气象数据");
            List<DownLoadFileInfo> downLoadFileInfoList = gfsDownloader.getFilesInfo();
            List<DownLoadFileInfo> finishedList = gfsDownloader.downloadAll(downLoadFileInfoList);
            List<DownLoadFileInfo> failedList = finishedList.stream().filter(fileInfo -> !fileInfo.isDownloadSuccess()).collect(Collectors.toList());
            if (!failedList.isEmpty()) {
                log.error("[schedule] 下载失败文件列表： {}", failedList);
                downloadFailedReport(gfsDownloader.getRefTimeStr(), finishedList, failedList);
                return false;
            }
            log.info("[schedule-end] 下载气象数据成功");
            return true;
        } catch (RuntimeException e) {
            log.error("[schedule-end] 下载气象数据失败", e);
            downloadFailedReport(gfsDownloader.getRefTimeStr(), e);
            return false;
        }
    }

    private boolean importToTableStore(){
        try {
            OffsetDateTime refTime = gfsDownloader.getRefTime();
            log.info("[schedule-start] 开始将数据导入数据库， 起报时间: {}", refTime);
            List<ImportFileInfo> importFileInfos = gfsDataImport.importData(refTime);
            List<ImportFileInfo> finishedList = importFileInfos.stream().filter(result -> result.isSuccess()).collect(Collectors.toList());
            List<ImportFileInfo> failedList = importFileInfos.stream().filter(result -> !result.isSuccess()).collect(Collectors.toList());
            if (!failedList.isEmpty()) {
                log.error("[schedule-end] 气象数据导入数据库失败, 文件列表： {} ", failedList);
                importFailedReport(gfsDownloader.getRefTimeStr(), finishedList, failedList);
                return false;
            }
            log.info("[schedule-end] 气象数据导入数据库成功");
            return true;
        } catch (Exception e) {
            log.error("[schedule-end] 气象数据导入数据库失败", e);
            importFailedReport(gfsDownloader.getRefTimeStr(),e);
            return false;
        }
    }


    public void clearExpiredData() {
        log.info("[schedule-start] 开始清理过期数据");
        File dataDir = new File(gfsDataConfig.getSaveRoot());
        if (!dataDir.exists()) {
            log.warn("[schedule] 文件夹 {} 不存在, 无法清理数据", dataDir);
            return;
        }
        OffsetDateTime now = OffsetDateTime.now();
        File[] dirs = dataDir.listFiles();
        for (File subDir : dirs) {
            OffsetDateTime refTime = LocalDateTime.parse(subDir.getName(), DateTimeFormatter.ofPattern(Constant.DATA_FOLDER_STRING)).atOffset(ZoneOffset.UTC);
            if (refTime.isBefore(now.minusDays(3))) {
                try {
                    FileUtils.deleteDirectory(subDir);
                    log.info("[schedule] 删除过期数据文件夹成功： {}", subDir.getName());
                } catch (IOException e) {
                    log.info("[schedule] 删除过期数据文件夹失败： {}", subDir.getName());
                }
            }
        }
        log.info("[schedule-end] 过期数据清理结束");
    }

    private void downloadFailedReport(String refTime, List<DownLoadFileInfo> finishedList, List<DownLoadFileInfo> failedList) {
        // 邮件
        Mail mail = new Mail();
        mail.setSubject("【航天飞鹏-气象服务】数据下载失败通知");
        mail.setTos(mailReceiver);
        StringBuilder sb = new StringBuilder();
        sb.append("数据下载失败通知\n 气象数据下载失败，请检查下载配置。\n");
        sb.append("起报时间: ").append(refTime).append("\n");
        sb.append("下载失败文件列表: \n");
        sb.append(failedList).append("\n");
        sb.append("下载成功文件列表: \n");
        sb.append(finishedList).append("\n");
        mail.setContent(sb.toString());
        Future<?> emailFuture = executorService.submit(() -> errorReportService.sendSimpleEmail(mail));

        // post
        ErrorReport errorReport = new ErrorReport();
        errorReport.setTime(OffsetDateTime.now().format(DateTimeFormatter.ISO_DATE_TIME));
        errorReport.setMessage("【航天飞鹏-气象服务】数据下载失败通知, 起报时间:" + refTime);
        Future<?> httpFuture = executorService.submit(() -> errorReportService.reportToRemote(errorReport));

        try {
            emailFuture.get();
            httpFuture.get();
        } catch (Exception ex) {
            log.error("[schedule] 异常报告过程错误: {}", ex.getMessage());
        }
    }

    private void downloadFailedReport(String refTime, Throwable e) {
        Mail mail = new Mail();
        mail.setSubject("【航天飞鹏-气象服务】数据下载失败通知");
        mail.setTos(mailReceiver);
        String sb = "数据下载失败通知\n 气象数据下载失败，请检查下载配置。\n" +
                "起报时间: " + refTime + "。\n";

        mail.setContent(sb + e);
        Future<?> emailFuture = executorService.submit(() -> errorReportService.sendSimpleEmail(mail));

        // post
        ErrorReport errorReport = new ErrorReport();
        errorReport.setTime(OffsetDateTime.now().format(DateTimeFormatter.ISO_DATE_TIME));
        errorReport.setMessage("【航天飞鹏-气象服务】数据下载失败通知, 起报时间:" + refTime);
        Future<?> httpFuture = executorService.submit(() -> errorReportService.reportToRemote(errorReport));

        try {
            emailFuture.get();
            httpFuture.get();
        } catch (Exception ex) {
            log.error("[schedule] 异常报告过程错误: {}", ex.getMessage());
        }
    }

    private void importFailedReport(String refTime, List<ImportFileInfo> finishedList, List<ImportFileInfo> failedList) {
        Mail mail = new Mail();
        mail.setSubject("【航天飞鹏-气象服务】数据入库失败通知");
        mail.setTos(mailReceiver);
        StringBuilder sb = new StringBuilder();
        sb.append("数据下载失败通知\n 数据入库下载失败，请检查。\n");
        sb.append("起报时间: ").append(refTime).append("\n");
        sb.append("失败文件列表: \n");
        sb.append(failedList).append("\n");
        sb.append("成功文件列表: \n");
        sb.append(finishedList).append("\n");
        mail.setContent(sb.toString());
        Future<?> emailFuture = executorService.submit(() -> errorReportService.sendSimpleEmail(mail));

        // post
        ErrorReport errorReport = new ErrorReport();
        errorReport.setTime(OffsetDateTime.now().format(DateTimeFormatter.ISO_DATE_TIME));
        errorReport.setMessage("【航天飞鹏-气象服务】数据入库失败通知, 起报时间:" + refTime);
        Future<?> httpFuture = executorService.submit(() -> errorReportService.reportToRemote(errorReport));

        try {
            emailFuture.get();
            httpFuture.get();
        } catch (Exception ex) {
            log.error("[schedule] 异常报告过程错误: {}", ex.getMessage());
        }
    }

    private void importFailedReport(String refTime, Throwable e) {
        Mail mail = new Mail();
        mail.setSubject("【航天飞鹏-气象服务】数据下载失败通知");
        mail.setTos(mailReceiver);
        String sb = "数据下载失败通知\n 气象数据下载失败，请检查下载配置。\n" +
                "起报时间: " + refTime + "。\n";
        mail.setContent(sb + e);
        Future<?> emailFuture = executorService.submit(() -> errorReportService.sendSimpleEmail(mail));

        // post
        ErrorReport errorReport = new ErrorReport();
        errorReport.setTime(OffsetDateTime.now().format(DateTimeFormatter.ISO_DATE_TIME));
        errorReport.setMessage("【航天飞鹏-气象服务】数据入库失败通知, 起报时间:" + refTime);
        Future<?> httpFuture = executorService.submit(() -> errorReportService.reportToRemote(errorReport));

        try {
            emailFuture.get();
            httpFuture.get();
        } catch (Exception ex) {
            log.error("[schedule] 异常报告过程错误: {}", ex.getMessage());
        }
    }

}
