package com.htfp.weather.web.config.aspect;

import com.htfp.weather.web.pojo.response.Result;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.aspectj.lang.JoinPoint;
import org.aspectj.lang.ProceedingJoinPoint;
import org.aspectj.lang.Signature;
import org.aspectj.lang.annotation.*;
import org.aspectj.lang.reflect.MethodSignature;
import org.springframework.core.annotation.Order;
import org.springframework.stereotype.Component;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;

import javax.servlet.http.HttpServletRequest;
import java.lang.reflect.Method;
import java.lang.reflect.Parameter;
import java.util.*;

/**
 * @Author : shiyi
 * @Date : 2024/6/12 14:12
 * @Description : 日志切面
 */
@Aspect
@Component
@Order(1)
@Slf4j
public class ControllerLogAspect {

    /**定义切点：指定controller包下面的公共方法且被ControllerLog注解修饰*/
    @Pointcut("execution(public * com.htfp.weather.web.controller.*.*(..))" +
            "&& @annotation(com.htfp.weather.web.config.aspect.ControllerLog)"
           )
    public void logPointCut() {
    }

    // @Before(value = "logPointCut()&& @annotation(controllerLog)")
    // public void doBefore(JoinPoint joinPoint, ControllerLog controllerLog) {
    //     log.info("[controllerLog] {} start: {}", controllerLog.info(), joinPoint.getSignature().toShortString());
    // }
    //
    // @AfterReturning(value = "logPointCut()&& @annotation(controllerLog)", returning = "ret")
    // public void doAfterReturning(JoinPoint joinPoint, Object ret, ControllerLog controllerLog) {
    //     log.info("[controllerLog] {} finished: {}, return: {}", controllerLog.info(), joinPoint.getSignature().toShortString(), ret);
    // }

    @AfterThrowing(value = "logPointCut()&& @annotation(controllerLog)", throwing = "cause")
    public void doAfterThrowing(JoinPoint joinPoint, Throwable cause, ControllerLog controllerLog){
        log.info("[controllerLog] {} failed: {}, errorMsg: {}", controllerLog.info(), joinPoint.getSignature().toShortString(), cause.getMessage());
    }

    @Around(value = "logPointCut()&& @annotation(controllerLog)")
    public Object doAround(ProceedingJoinPoint joinPoint, ControllerLog controllerLog) throws Throwable {
        //获取当前请求对象
        ServletRequestAttributes attributes = (ServletRequestAttributes) RequestContextHolder.getRequestAttributes();
        if (attributes == null) {
            return Result.error(new RuntimeException("切点处理异常: " + this.getClass().getName()));
        }
        HttpServletRequest request = attributes.getRequest();
        Signature signature = joinPoint.getSignature();
        MethodSignature methodSignature = (MethodSignature) signature;
        Method method = methodSignature.getMethod();
        //记录请求信息

        Object parameter = getParameter(method, joinPoint.getArgs());

        log.info("[controllerLog] {} start: {}, request params: {}", controllerLog.info(), signature.toShortString(), parameter);
        long startTime = System.currentTimeMillis();
        // 调用目标切点方法
        Object result = joinPoint.proceed();
        long endTime = System.currentTimeMillis();
        int timeCost = (int) (endTime - startTime);

        if (controllerLog.isSaveToDatabase()) {
            WebLog webLog = new WebLog();
            webLog.setDescription(controllerLog.info());
            webLog.setIp(request.getRemoteUser());
            webLog.setRequestMethod(request.getMethod());
            webLog.setParameter(parameter);
            webLog.setResult(result);
            webLog.setTimeCost(timeCost);
            webLog.setStartTime(startTime);
            webLog.setUri(request.getRequestURI());
            webLog.setUrl(request.getRequestURL().toString());
        }
        log.debug("[controllerLog] {} finished ({}ms): {}, return: {}", controllerLog.info(), timeCost, signature.toShortString(), result);
        log.info("[controllerLog] {} finished ({}ms): {}", controllerLog.info(), timeCost, signature.toShortString());

        // 返回值将由controller返回
        return result;
    }
    /**
     * 根据方法和传入的参数获取请求参数
     */
    private Object getParameter(Method method, Object[] args) {
        List<Object> argList = new ArrayList<>();
        Parameter[] parameters = method.getParameters();
        for (int i = 0; i < parameters.length; i++) {
            //将RequestBody注解修饰的参数作为请求参数
            RequestBody requestBody = parameters[i].getAnnotation(RequestBody.class);
            if (requestBody != null) {
                argList.add(args[i]);
            }
            //将RequestParam注解修饰的参数作为请求参数
            RequestParam requestParam = parameters[i].getAnnotation(RequestParam.class);
            if (requestParam != null) {
                Map<String, Object> map = new HashMap<>();
                String key = parameters[i].getName();
                if (!StringUtils.isEmpty(requestParam.value())) {
                    key = requestParam.value();
                }
                map.put(key, args[i]);
                argList.add(map);
            }
            // else {
            //     argList.add(args[i]);
            // }
        }
        if (argList.isEmpty()) {
            return null;
        } else if (argList.size() == 1) {
            return argList.get(0);
        } else {
            return argList;
        }
    }
}
