package com.htfp.weather.web.controller;

import com.htfp.weather.download.DownLoadFileInfo;
import com.htfp.weather.download.gfs.GfsDataConfig;
import com.htfp.weather.griddata.common.TableConfig;
import com.htfp.weather.griddata.operation.GfsDataImport;
import com.htfp.weather.griddata.operation.GfsDataImport.ImportFileInfo;
import com.htfp.weather.info.Constant;
import com.htfp.weather.utils.DateTimeUtils;
import com.htfp.weather.web.config.aspect.ControllerLog;
import com.htfp.weather.web.exception.AppException;
import com.htfp.weather.web.exception.ErrorCode;
import com.htfp.weather.web.pojo.request.GfsConfigUpdate;
import com.htfp.weather.web.pojo.request.TableConfigUpdate;
import com.htfp.weather.web.pojo.response.DownloadResult;
import com.htfp.weather.web.pojo.response.ImportResult;
import com.htfp.weather.web.pojo.response.Result;
import com.htfp.weather.web.service.FileService;
import com.htfp.weather.web.service.GfsDataServiceImpl;
import org.apache.commons.lang3.StringUtils;
import org.jetbrains.annotations.NotNull;
import org.springframework.util.CollectionUtils;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import java.io.File;
import java.io.IOException;
import java.time.OffsetDateTime;
import java.time.LocalDateTime;
import java.time.ZoneOffset;
import java.time.format.DateTimeFormatter;
import java.util.List;
import java.util.Map;

/**
 * @Author : shiyi
 * @Date : 2024/4/21 15:56
 * @Description : 配置文件管理
 */
@CrossOrigin
@RestController
@RequestMapping("/htfp/weather/config")
public class ConfigController {
    @Resource
    TableConfig tableConfig;
    @Resource
    GfsDataConfig gfsDataConfig;
    @Resource
    GfsDataImport gfsDataImport;
    @Resource
    FileService fileService;
    @Resource
    GfsDataServiceImpl gfsDataService;
    private final String SECRET = "htfpweather";

    private void validSecret(String secret) {
        if (StringUtils.isEmpty(secret) || !SECRET.equals(secret)) {
            throw new AppException(ErrorCode.SECRET_ERROR);
        }
    }

    /**
     * 查询tablestore数据库配置
     */
    @RequestMapping("/queryDatabaseConfig")
    @ControllerLog(info = "查询tablestore数据库配置")
    public Result queryDatabaseConfig() {
        return Result.success(tableConfig);
    }

    /**
     * 更新tablestore数据库配置
     */
    @PostMapping("/updateDatabaseConfig")
    @ControllerLog(info = "更新tablestore数据库配置")
    public Result updateDatabaseConfig(@RequestBody TableConfigUpdate request) {
        String secret = request.getSecret();
        TableConfig updateConfig = request.getConfig();
        validSecret(secret);
        if (updateConfig == null) {
            throw new AppException(ErrorCode.CONFIG_ERROR, "配置文件不能为空");
        }
        updateConfig.valid();
        try {
            tableConfig.writeConfig(updateConfig);
        } catch (IOException e) {
            throw new AppException(ErrorCode.CONFIG_ERROR, e);
        }
        return Result.success();
    }

    /**
     * 查询GFS下载配置
     */
    @RequestMapping("/queryDataSourceConfig")
    @ControllerLog(info = "查询GFS下载配置")
    public Result queryDataSourceConfig() {
        return Result.success(gfsDataConfig);
    }

    /**
     * 更新GFS下载配置
     */
    @PostMapping("/updateDataSourceConfig")
    @ControllerLog(info = "更新GFS下载配置")
    public Result updateDataSourceConfig(@Validated @RequestBody GfsConfigUpdate request) {
        String secret = request.getSecret();
        GfsDataConfig updateConfig = request.getConfig();
        validSecret(secret);
        if (updateConfig == null) {
            throw new AppException(ErrorCode.CONFIG_ERROR, "配置文件不能为空");
        }
        updateConfig.valid();
        try {
            gfsDataConfig.writeConfig(updateConfig);
        } catch (IOException e) {
            throw new AppException(ErrorCode.CONFIG_ERROR, e);
        }
        return Result.success();
    }

    /**
     * 下载当前时刻未来24小时的数据文件
     */
    @PostMapping("/downloadAllFiles")
    @ControllerLog(info = "下载当前时刻未来24小时的数据文件")
    public Result downloadAllFiles(@RequestBody Map<String, String> params) {
        String secret = params.get("secret");
        validSecret(secret);
        // List<FileInfo> allCompleted = false;
        try {
            DownloadResult data = gfsDataService.downloadNowAllFile();
            List<DownLoadFileInfo> failedList = data.getFailedList();
            if (CollectionUtils.isEmpty(failedList)) {
                return Result.success(data);
            } else {
                return new Result(false, ErrorCode.DOWNLOAD_ERROR.getCode(), ErrorCode.DOWNLOAD_ERROR.getMsg() + ":下载未全部完成", data);
            }
        } catch (AppException e) {
            return Result.error(ErrorCode.DOWNLOAD_START_ERROR, e.getMessage());
        }
    }

    /**
     * 下载指定时刻文件
     */
    @PostMapping("/downloadSingleFile")
    @ControllerLog(info = "下载指定时刻文件")
    public Result downloadSingleFile(@RequestBody Map<String, String> params) {
        String secret = params.get("secret");
        validSecret(secret);
        OffsetDateTime offsetDateTime = OffsetDateTime.parse(params.get("time"), Constant.API_TIME_FORMATTER);
        DownLoadFileInfo downLoadFileInfo = gfsDataService.downloadSingleFile(offsetDateTime);
        return Result.success(downLoadFileInfo);
    }

    /**
     * 根据起报时间将数据导入数据库
     */
    @PostMapping("/importToTablestoreByReferenceTime")
    @ControllerLog(info = "根据起报时间将数据导入数据库")
    public Result importData(@RequestBody Map<String, String> params) {
        String secret = params.get("secret");
        validSecret(secret);
        OffsetDateTime time = OffsetDateTime.parse(params.get("time"), Constant.API_TIME_FORMATTER);
        return getImportResponse(time);
    }

    /**
     * 下载并导入
     */
    @PostMapping("/downloadAndImport")
    @ControllerLog(info = "下载并导入")
    public Result downloadAndImport(@RequestBody Map<String, String> params) {
        Result downloadResponse = downloadAllFiles(params);
        if (downloadResponse.isSuccess()) {
            DownloadResult data = (DownloadResult) downloadResponse.getData();
            String refTimeStr = data.getSuccessList().get(0).getRefTimeStr();
            OffsetDateTime time = OffsetDateTime.of(LocalDateTime.parse(refTimeStr, DateTimeFormatter.ofPattern(Constant.UTC_TIME_STRING)), ZoneOffset.UTC);
            return getImportResponse(time);
        } else {
            return downloadResponse;
        }
    }

    @NotNull
    private Result getImportResponse(OffsetDateTime time) {
        try {
            ImportResult importResult = gfsDataService.importData(time);
            List<ImportFileInfo> failedList = importResult.getFailedList();
            if (CollectionUtils.isEmpty(failedList)) {
                return Result.success(importResult);
            } else {
                return new Result(false, ErrorCode.TABLESTORE_IMPORT_ERROR.getCode(), ErrorCode.TABLESTORE_IMPORT_ERROR.getMsg() + ": 数据导入未全部完成", importResult);
            }
        } catch (AppException e) {
            return Result.error(e.getErrorCode());
        } catch (Exception e) {
            return Result.error(e);
        }
    }

    @PostMapping("queryAllDataDir")
    @ControllerLog(info = "查询所有数据文件夹")
    public Result queryAllDataDir(@RequestBody Map<String, String> params) {
        String secret = params.get("secret");
        validSecret(secret);
        File dataRoot = new File(gfsDataConfig.getSaveRoot());
        List<String> subDirList = fileService.getSubDirList(dataRoot);
        return Result.success(subDirList);
    }

    /**
     * 查询指定起报时间文件夹中的所有文件名
     */
    @PostMapping("/queryAllFileByReferenceTime")
    @ControllerLog(info = "查询指定起报时间文件夹中的所有文件名")
    public Result queryAllFileByReferenceTime(@RequestBody Map<String, String> params) {
        String secret = params.get("secret");
        validSecret(secret);
        OffsetDateTime time = OffsetDateTime.parse(params.get("time"), Constant.API_TIME_FORMATTER);
        String timeStr = DateTimeUtils.getUTCDateTime(time).format(DateTimeFormatter.ofPattern(Constant.DATA_FOLDER_STRING));
        File dataDir = new File(gfsDataConfig.getSaveRoot(), timeStr);

        // 查找目录下的文件
        List<com.htfp.weather.web.pojo.response.FileInfo> fileList = fileService.getFileList(dataDir);
        return Result.success(fileList);
    }

    /**
     * 删除指定起报时间对应的文件夹
     */
    @PostMapping("/deleteDirByReferenceTime")
    @ControllerLog(info = "删除指定起报时间对应的文件夹")
    public Result deleteDirByReferenceTime(@RequestBody Map<String, String> params) {
        String secret = params.get("secret");
        validSecret(secret);
        OffsetDateTime time = OffsetDateTime.parse(params.get("time"), Constant.API_TIME_FORMATTER);
        String timeStr = DateTimeUtils.getUTCDateTime(time).format(DateTimeFormatter.ofPattern(Constant.DATA_FOLDER_STRING));
        File dataDir = new File(gfsDataConfig.getSaveRoot(), timeStr);
        if (gfsDataImport.isImporting()) {
            return Result.error(ErrorCode.CAN_NOT_DELETE_WHEN_IMPORT_OR_DOWNLOAD);
        }
        try {
            fileService.deleteDir(dataDir);
            return Result.success();
        } catch (IOException e) {
            return Result.error(e);
        }
    }

    /**
     * 删除指定文件
     */
    @PostMapping("/deleteTargetFile")
    @ControllerLog(info = "删除指定文件")
    public Result deleteTargetFile(@RequestBody Map<String, String> params) {
        String secret = params.get("secret");
        validSecret(secret);
        String filePath = params.get("filePath");
        File file = new File(filePath);
        if (gfsDataImport.isImporting()) {
            return Result.error(ErrorCode.CAN_NOT_DELETE_WHEN_IMPORT_OR_DOWNLOAD);
        }
        try {
            fileService.deleteFile(file);
            return Result.success();
        } catch (IOException e) {
            return Result.error(e);
        }
    }
}
