package com.htfp.weather.web.controller;

import com.htfp.weather.utils.DateTimeUtils;
import com.htfp.weather.web.config.aspect.ControllerLog;
import com.htfp.weather.web.exception.AppException;
import com.htfp.weather.web.exception.ErrorCode;
import com.htfp.weather.web.pojo.request.*;
import com.htfp.weather.web.pojo.response.*;
import com.htfp.weather.web.service.GfsDataServiceImpl;
import lombok.extern.slf4j.Slf4j;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import java.time.OffsetDateTime;
import java.util.ArrayList;
import java.util.List;

/**
 * @Author : shiyi
 * @Date : 2024/5/8 15:15
 * @Description : 高空天气查询接口
 */
@CrossOrigin
@RestController
@Slf4j
@RequestMapping("/htfp/weather/upper/")
public class UpperWeatherController {

    @Resource(name = "tablestore-gfs")
    GfsDataServiceImpl gfsDataService;

    @PostMapping("/queryUpperNowWeather")
    @ControllerLog(info = "高空实时气象信息查询")
    public Result queryUpperNowWeather(@Validated @RequestBody Position3D position3D) {
        double latitude = position3D.getLatitude();
        double longitude = position3D.getLongitude();
        int level = position3D.getLevel();
        NowWeatherStatus nowWeather = gfsDataService.getNowWeather(level, latitude, longitude);
        return Result.success(nowWeather);
    }

    @PostMapping("/queryUpperNowWeatherInMultiPoints")
    @ControllerLog(info = "高空航点实时气象信息查询")
    public Result queryUpperNowWeatherInMultiPoints(@Validated @RequestBody MultiPointsRequest multiPointsRequest) {
        List<Double> latitudeList = new ArrayList<>();
        List<Double> longitudeList = new ArrayList<>();
        for (Position2D position : multiPointsRequest.getPointList()) {
            Double latitude = position.getLatitude();
            Double longitude = position.getLongitude();
            if (latitude != null && longitude != null) {
                latitudeList.add(latitude);
                longitudeList.add(longitude);
            } else {
                throw new AppException(ErrorCode.MULTI_INDEX_ERROR);
            }
        }
        int level = multiPointsRequest.getLevel().intValue();
        List<NowWeatherStatus> nowWeatherByMultiPoint = gfsDataService.getNowWeatherByMultiPoint(level, latitudeList, longitudeList);
        return Result.success(nowWeatherByMultiPoint);
    }

    @PostMapping("/queryUpperForecast")
    @ControllerLog(info = "高空24小时预报结果查询")
    public Result queryUpperForecast(@Validated @RequestBody Position3D position3D) {
        double latitude = position3D.getLatitude();
        double longitude = position3D.getLongitude();
        int level = position3D.getLevel();
        TimeSeriesDataset forecastSeries = gfsDataService.getForecastSeries(level, latitude, longitude);
        return Result.success(forecastSeries);
    }

    @PostMapping("/queryUpperForecastInMultiPoints")
    @ControllerLog(info = "高空航点24小时预报结果查询")
    public Result queryUpperForecastInMultiPoints(@Validated @RequestBody MultiPointsRequest multiPointsRequest) {
        List<Double> latitudeList = new ArrayList<>();
        List<Double> longitudeList = new ArrayList<>();
        for (Position2D position : multiPointsRequest.getPointList()) {
            Double latitude = position.getLatitude();
            Double longitude = position.getLongitude();
            if (latitude != null && longitude != null) {
                latitudeList.add(latitude);
                longitudeList.add(longitude);
            } else {
                throw new AppException(ErrorCode.MULTI_INDEX_ERROR);
            }
        }
        int level = multiPointsRequest.getLevel().intValue();
        List<TimeSeriesDataset> nowWeatherByMultiPoint = gfsDataService.getForecastSeriesByMultiPoint(latitudeList, longitudeList, level);
        return Result.success(nowWeatherByMultiPoint);
    }


    /**
     * 查询指定变量随气压的分布
     */
    @RequestMapping("/queryProfileByVariableAndPressure")
    @ControllerLog(info = "指定变量高度廓线（气压坐标）查询")
    public Result queryProfileByVariableAndPressure(@Validated @RequestBody ProfileRequest profileRequest) {
        OffsetDateTime time = OffsetDateTime.parse(profileRequest.getTime());
        OffsetDateTime utcDateTime = DateTimeUtils.getUTCDateTime(time);
        String variableName = profileRequest.getVariableName();
        double latitude = profileRequest.getLatitude();
        double longitude = profileRequest.getLongitude();

        ProfileResponse profileResponse = gfsDataService.getProfileByVariableAndPressure(variableName, utcDateTime, latitude, longitude);
        return Result.success(profileResponse);
    }

    /**
     * 查询全部变量随气压的分布
     */
    @RequestMapping("/queryProfileByPressure")
    @ControllerLog(info = "全部变量高度廓线（气压坐标）查询")
    public Result queryProfileByAndPressure(@Validated @RequestBody ProfileRequest profileRequest) {
        OffsetDateTime time = OffsetDateTime.parse(profileRequest.getTime());
        OffsetDateTime utcDateTime = DateTimeUtils.getUTCDateTime(time);
        double latitude = profileRequest.getLatitude();
        double longitude = profileRequest.getLongitude();

        ProfileDataset profile = gfsDataService.getProfileByPressure(utcDateTime, latitude, longitude);
        return Result.success(profile);
    }

    /**
     * 查询指定近地面变量随高度的分布
     */
    @RequestMapping("/queryProfileByVariableAndNearSurfaceHeight")
    @ControllerLog(info = "指定变量高度廓线（近地面高度坐标）查询")
    public Result queryProfileByVariableAndNearSurfaceHeight(@Validated @RequestBody ProfileRequest profileRequest) {
        OffsetDateTime time = OffsetDateTime.parse(profileRequest.getTime());
        OffsetDateTime utcDateTime = DateTimeUtils.getUTCDateTime(time);
        String variableName = profileRequest.getVariableName();
        double latitude = profileRequest.getLatitude();
        double longitude = profileRequest.getLongitude();

        ProfileResponse profile = gfsDataService.getProfileByVariableAndNearSurfaceHeight(variableName, utcDateTime, latitude, longitude);
        return Result.success(profile);
    }

    /**
     * 查询全部近地面变量随高度的分布
     */
    @RequestMapping("/queryProfileByNearSurfaceHeight")
    @ControllerLog(info = "全部变量高度廓线（近地面高度坐标）查询")
    public Result queryProfileByNearSurfaceHeight(@Validated @RequestBody ProfileRequest profileRequest) {
        OffsetDateTime time = OffsetDateTime.parse(profileRequest.getTime());
        OffsetDateTime utcDateTime = DateTimeUtils.getUTCDateTime(time);
        double latitude = profileRequest.getLatitude();
        double longitude = profileRequest.getLongitude();

        log.info("[data-server] 高度廓线（近地面高度坐标）查询 start: param={}", profileRequest);
        ProfileDataset profile = gfsDataService.getProfileByNearSurfaceHeight(utcDateTime, latitude, longitude);
        return Result.success(profile);
    }

    @PostMapping("/queryPlaneGrid")
    @ControllerLog(info = "平面网格数据查询")
    public Result queryPlaneGrid(@Validated @RequestBody PlaneRequest planeRequest) {
        planeRequest.valid();
        String variableName = planeRequest.getVariableName();
        OffsetDateTime time = OffsetDateTime.parse(planeRequest.getTime());
        OffsetDateTime utcDateTime = DateTimeUtils.getUTCDateTime(time);
        int level = planeRequest.getLevel();
        double minLat = planeRequest.getMinLatitude();
        double maxLat = planeRequest.getMaxLatitude();
        double minLon = planeRequest.getMinLongitude();
        double maxLon = planeRequest.getMaxLongitude();

        PlaneResponse forecastSeries = gfsDataService.getPlane(utcDateTime, variableName, level, minLat, maxLat, minLon, maxLon);
        return Result.success(forecastSeries);
    }

}
