package com.htfp.weather.web.service.surfaceapi;

import com.htfp.weather.info.Constant;
import com.htfp.weather.utils.DateTimeUtils;
import com.htfp.weather.utils.JSONUtils;
import com.htfp.weather.web.exception.AppException;
import com.htfp.weather.web.exception.ErrorCode;
import com.htfp.weather.web.pojo.cma.AntiGeoCodeResponse;
import com.htfp.weather.web.pojo.cma.AntiGeoCodeResponse.Result.AddressComponent;
import com.htfp.weather.web.pojo.cma.CmaWarning;
import com.htfp.weather.web.pojo.response.NowWeatherStatus;
import com.htfp.weather.web.pojo.response.SurfaceWeatherWarning;
import com.htfp.weather.web.pojo.response.TimeSeriesDataset;
import lombok.Setter;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Service;

import javax.annotation.PostConstruct;
import java.time.OffsetDateTime;
import java.time.format.DateTimeFormatter;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;

import static com.htfp.weather.utils.HttpClientUtils.sendGet;

/**
 * @Author : shiyi
 * @Date : 2024/7/2 14:02
 * @Description : 中央气象台相关数据
 */
@Slf4j
@Service("cma")
public class CmaServiceImpl implements ISurfaceDataService{
    @Value("${tianditu.key}") @Setter
    private String keyOfTianDiTu;
    private Map<String, List<SurfaceWeatherWarning>> warningCache = new ConcurrentHashMap<>();

    @Override
    public NowWeatherStatus getNowSurfaceWeatherStatus(double lat, double lon) throws Exception {
        return null;
    }

    @Override
    public TimeSeriesDataset getForecastSeries(double lat, double lon) throws Exception {
        return null;
    }

    @Override
    public List<SurfaceWeatherWarning> getSurfaceWarning(double lat, double lon) throws Exception {
        String countyId = getCountyIdByLatLon(lat, lon);
        return getSurfaceWarningByCountyCode(countyId);
    }

    public List<SurfaceWeatherWarning> getSurfaceWarningByCountyCode(String countyCode) {
        if (warningCache.containsKey(countyCode)) {
            return warningCache.get(countyCode);
        } else {
            return new ArrayList<>();
        }
        // else {
        //     // 如果缓存中没有，则先查一遍
        //     updateCmaWarning(countyCode);
        //     return warningCache.get(countyCode);
        // }

    }

    /**根据经纬度获取县级行政区代码*/
    public String getCountyIdByLatLon(double lat, double lon) throws Exception {
        String url = "http://api.tianditu.gov.cn/geocoder";
        HashMap<String, String> params = new HashMap<>();
        params.put("postStr", String.format("{'lon':%.6f,'lat':%.6f,'ver':1}", lon, lat));
        params.put("type", "geocode");
        params.put("tk", keyOfTianDiTu);
        AntiGeoCodeResponse antiGeoCodeResponse = sendGet(url, params, AntiGeoCodeResponse.class);
        if (antiGeoCodeResponse.getResult() == null){
            throw new AppException(ErrorCode.TIANDITU_REQUEST_ERROR, "获取行政区划信息失败：" + antiGeoCodeResponse.getMsg());
        }
        AddressComponent addressComponent = antiGeoCodeResponse.getResult().getAddressComponent();
        String countyCode = addressComponent.getCountyCode();
        if (StringUtils.isEmpty(countyCode)) {
            String info;
            info = addressComponent.getInfo();
            throw new AppException(ErrorCode.TIANDITU_DATA_PARSE_ERROR, "获取行政区代码失败：" + info);
        } else {
            return countyCode.substring(countyCode.length()-6);
        }
    }
    /**每半小时定时更新全国的气象预警数据；
     * 存在数据更新不及时，例如地震龙卷飑线等短时强灾害
     * */
    @Scheduled(cron = "0 20,50 * * * ?")
    @PostConstruct
    public void updateCmaWarning() {
        updateCmaWarning(null); // 更新全国预警数据
    }

    /**更新某县区所在省的全部预警数据
     * @param countyCode 县级行政区代码*/
    public void updateCmaWarning(String countyCode) {
        String url = "https://data.cma.cn/dataGis/disasterWarning/getWarningDataByCnty";
        // 6小时前，12小时后
        OffsetDateTime startTime = DateTimeUtils.offsetDateTimeToSystemZone(OffsetDateTime.now()).minusHours(6);
        OffsetDateTime endTime = startTime.plusHours(12);
        DateTimeFormatter formatter = DateTimeFormatter.ofPattern("yyyy-MM-dd+HH:mm:ss");
        HashMap<String, String> params = new HashMap<>();
        params.put("startTime", startTime.format(formatter));
        params.put("endTime", endTime.format(formatter));
        params.put("provinceCode", countyCode);
        log.info("更新预警数据: {}", params);
        try {
            if (params.get("provinceCode") == null) {
                warningCache.clear();
            }
            String jsonStr = sendGet(url, params);
            List<CmaWarning> cmaWarningList = JSONUtils.json2list(jsonStr, CmaWarning.class);
            for (CmaWarning cmaWarning : cmaWarningList) {
                String countyId = cmaWarning.getAreaId();
                String dataid = cmaWarning.getDataid();
                SurfaceWeatherWarning surfaceWeatherWarning = buildSurfaceWeatherWarning(cmaWarning);
                if (! warningCache.containsKey(countyId)) {
                    warningCache.put(countyId, new ArrayList<>());
                }
                List<SurfaceWeatherWarning> countyCacheList = warningCache.get(countyId);
                if (countyCacheList.stream().noneMatch(w -> dataid.equals(w.getDataid()))) {
                    // 该数据未缓存过
                    warningCache.get(countyId).add(surfaceWeatherWarning);
                }
            }
        } catch (Exception e) {
            log.error("[中央气象台] 预警信息请求结果处理错误， 预警数据更新失败, {}", e.getMessage());
        }
    }

    private SurfaceWeatherWarning buildSurfaceWeatherWarning(CmaWarning cmaWarning) {
        SurfaceWeatherWarning surfaceWeatherWarning = new SurfaceWeatherWarning();
        surfaceWeatherWarning.setSource(cmaWarning.getSender());
        surfaceWeatherWarning.setSeverityColor(cmaWarning.getSignallevelcode().toUpperCase());
        surfaceWeatherWarning.setText(cmaWarning.getIssuecontent());
        String startTime = cmaWarning.getTime(); // NOTE 2024/7/2: 中央气象台默认使用 GMT+8 北京时
        if (StringUtils.isNotEmpty(startTime)) {
            surfaceWeatherWarning.setPubTime(OffsetDateTime.parse(startTime + "+08:00", DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ssxxx")).format(Constant.API_TIME_FORMATTER)
            );
        }
        String endTime = cmaWarning.getExpires();
        if (StringUtils.isNotEmpty(endTime)) {
            surfaceWeatherWarning.setEndTime(OffsetDateTime.parse(endTime + "+08:00", DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ssxxx")).format(Constant.API_TIME_FORMATTER)
            );
        }
        surfaceWeatherWarning.setTypeName(cmaWarning.getCodename());
        surfaceWeatherWarning.setTitle(String.format("%s发布:%s%s预警", cmaWarning.getSender(), cmaWarning.getCodename(), cmaWarning.getSignallevel()));
        surfaceWeatherWarning.setDataid(cmaWarning.getDataid());
        return surfaceWeatherWarning;
    }
}
